@@ -134,9 +134,10 @@ See L<perlguts/Autoloading with XSUBs>.
 #define CVf_HASEVAL	0x4000	/* contains string eval  */
 #define CVf_NAMED	0x8000  /* Has a name HEK */
 #define CVf_LEXICAL	0x10000 /* Omit package from name */
+#define CVf_ANONCONST	0x20000 /* :const - create anonconst op */
 
 /* This symbol for optimised communication between toke.c and op.c: */
-#define CVf_BUILTIN_ATTRS	(CVf_METHOD|CVf_LVALUE)
+#define CVf_BUILTIN_ATTRS	(CVf_METHOD|CVf_LVALUE|CVf_ANONCONST)
 
 #define CvCLONE(cv)		(CvFLAGS(cv) & CVf_CLONE)
 #define CvCLONE_on(cv)		(CvFLAGS(cv) |= CVf_CLONE)
@@ -219,6 +220,10 @@ See L<perlguts/Autoloading with XSUBs>.
 #define CvLEXICAL_on(cv)	(CvFLAGS(cv) |= CVf_LEXICAL)
 #define CvLEXICAL_off(cv)	(CvFLAGS(cv) &= ~CVf_LEXICAL)
 
+#define CvANONCONST(cv)		(CvFLAGS(cv) & CVf_ANONCONST)
+#define CvANONCONST_on(cv)	(CvFLAGS(cv) |= CVf_ANONCONST)
+#define CvANONCONST_off(cv)	(CvFLAGS(cv) &= ~CVf_ANONCONST)
+
 /* Flags for newXS_flags  */
 #define XS_DYNAMIC_FILENAME	0x01	/* The filename isn't static  */
 
@@ -259,7 +264,7 @@ There is a further complication with non-closure anonymous subs (i.e. those
 that do not refer to any lexicals outside that sub).  In this case, the
 anonymous prototype is shared rather than being cloned.  This has the
 consequence that the parent may be freed while there are still active
-children, eg
+children, I<e.g.>,
 
     BEGIN { $a = sub { eval '$x' } }
 
@@ -305,11 +310,5 @@ typedef OP *(*Perl_call_checker)(pTHX_ OP *, GV *, SV *);
 #endif
 
 /*
- * Local variables:
- * c-indentation-style: bsd
- * c-basic-offset: 4
- * indent-tabs-mode: nil
- * End:
- *
  * ex: set ts=8 sts=4 sw=4 et:
  */
