@@ -7,7 +7,7 @@ use strict;
 
 # Package globals
 @ISA = ( 'DynaLoader' );
-$VERSION = '1.04';
+$VERSION = '1.07';
 my(%Locsyms) = ( ':ID' => 'LOCAL' );
 my(%Gblsyms) = ( ':ID' => 'GLOBAL');
 my $DoCache = 1;
@@ -18,6 +18,8 @@ my $Cache_set = 0;
 
 sub new {
   my($pkg,$type) = @_;
+  $type ||= 'LOCAL';
+  $type = 'LOCAL' unless $type eq 'GLOBAL';
   bless { TYPE => $type }, $pkg;
 }
 
@@ -73,7 +75,7 @@ sub clearcache {
 #====> TIEHASH methods
 
 sub TIEHASH {
-  $_[0]->new(@_);
+  shift->new(@_);
 }
 
 sub FETCH {
@@ -159,8 +161,9 @@ VMS::DCLsym - Perl extension to manipulate DCL symbols
 
   $handle = new VMS::DCLsym;
   $value = $handle->getsym($name);
-  $handle->setsym($name,$value,'GLOBAL') or die "Can't create symbol: $!\n";
-  $handle->delsym($name,'LOCAL') or die "Can't delete symbol: $!\n";
+  $handle->setsym($name, $value, 'GLOBAL')
+      or die "Can't create symbol: $!\n";
+  $handle->delsym($name, 'LOCAL') or die "Can't delete symbol: $!\n";
   $handle->clearcache();
 
 =head1 DESCRIPTION
@@ -183,8 +186,8 @@ defines a new symbol (or overwrites the old value of an existing symbol), and
 deleting an element deletes the corresponding symbol.  Setting an element to
 C<undef>, or C<undef>ing it directly, sets the corresponding symbol to the null
 string. You may also read the special keys ':GLOBAL' and ':LOCAL' to find out
-whether a default symbol table has been specified for this hash (see C<table>
-below), or set either or these keys to specify a default symbol table.
+whether a default symbol table has been specified for this hash (see the next
+paragraph), or set either or these keys to specify a default symbol table.
 
 When you call the C<tie> function to bind an associative array to this package,
 you may specify as an optional argument the symbol table in which you wish to
@@ -262,7 +265,7 @@ Charles Bailey  bailey@newman.upenn.edu
 
 =head1 VERSION
 
-1.01  08-Dec-1996
+1.05  12-Feb-2011
 
 =head1 BUGS
 
