@@ -44,8 +44,8 @@ sub full_name ($$) { # Returns the function name with potentially the
 		     # prefixes 'S_' or 'Perl_'
     my ($func, $flags) = @_;
 
+    return "Perl_$func" if $flags =~ /p/;
     return "S_$func" if $flags =~ /[si]/;
-    return "Perl_$func" if $flags =~ /[bp]/;
     return $func;
 }
 
@@ -75,11 +75,14 @@ my ($embed, $core, $ext, $api) = setup_embed();
 	}
 
 	my ($flags,$retval,$plain_func,@args) = @$_;
+	if ($flags =~ / ( [^AabDdEfiMmnOoPpRrsUXx] ) /x) {
+	    warn "flag $1 is not legal (for function $plain_func)";
+	}
 	my @nonnull;
 	my $has_context = ( $flags !~ /n/ );
 	my $never_returns = ( $flags =~ /r/ );
-	my $commented_out = ( $flags =~ /m/ );
 	my $binarycompat = ( $flags =~ /b/ );
+	my $commented_out = ( ! $binarycompat && $flags =~ /m/ );
 	my $is_malloc = ( $flags =~ /a/ );
 	my $can_ignore = ( $flags !~ /R/ ) && !$is_malloc;
 	my @names_of_nn;
@@ -89,12 +92,8 @@ my ($embed, $core, $ext, $api) = setup_embed();
 	    warn "It is nonsensical to require the return value of a void function ($plain_func) to be checked";
 	}
 
-	my $scope_type_flag_count = 0;
-	$scope_type_flag_count++ if $flags =~ /s/;
-	$scope_type_flag_count++ if $flags =~ /i/;
-	$scope_type_flag_count++ if $flags =~ /p/;
-	warn "$plain_func: i, p, and s flags are all mutually exclusive"
-						   if $scope_type_flag_count > 1;
+	warn "$plain_func: s flag is mutually exclusive from the i and p plags"
+					    if $flags =~ /s/ && $flags =~ /[ip]/;
 	my $splint_flags = "";
 	if ( $SPLINT && !$commented_out ) {
 	    $splint_flags .= '/*@noreturn@*/ ' if $never_returns;
@@ -122,7 +121,9 @@ my ($embed, $core, $ext, $api) = setup_embed();
 	    }
 	}
 	$func = full_name($plain_func, $flags);
-	$ret = "$retval\t$func(";
+	$ret = "";
+	$ret .= "#ifndef NO_MATHOMS\n" if $binarycompat;
+	$ret .= "$retval\t$func(";
 	if ( $has_context ) {
 	    $ret .= @args ? "pTHX_ " : "pTHX";
 	}
@@ -216,6 +217,7 @@ my ($embed, $core, $ext, $api) = setup_embed();
 	    $ret .= "\n#define PERL_ARGS_ASSERT_\U$plain_func\E\t\\\n\t"
 		. join '; ', map "assert($_)", @names_of_nn;
 	}
+	$ret .= "\n#endif" if $binarycompat;
 	$ret .= @attrs ? "\n\n" : "\n";
 
 	print $pr $ret;
@@ -300,7 +302,9 @@ sub embed_h {
 	unless ($flags =~ /[om]/) {
 	    my $args = scalar @args;
 	    if ($flags =~ /n/) {
-		$ret = hide($func, full_name($func, $flags));
+		my $full_name = full_name($func, $flags);
+		next if $full_name eq $func;	# Don't output a no-op.
+		$ret = hide($func, $full_name);
 	    }
 	    elsif ($args and $args[$args-1] =~ /\.\.\./) {
 		if ($flags =~ /p/) {
@@ -319,6 +323,7 @@ sub embed_h {
 		$ret .= "_ " if $alist;
 		$ret .= $alist . ")\n";
 	    }
+	    $ret = "#ifndef NO_MATHOMS\n$ret#endif\n" if $flags =~ /b/;
 	}
 	$lines .= $ret;
     }
@@ -476,9 +481,11 @@ END
 
 for $sym (@globvar) {
     print $em "#ifdef OS2\n" if $sym eq 'sh_path';
+    print $em "#ifdef __VMS\n" if $sym eq 'perllib_sep';
     print $em multon($sym,   'G','my_vars->');
     print $em multon("G$sym",'', 'my_vars->');
     print $em "#endif\n" if $sym eq 'sh_path';
+    print $em "#endif\n" if $sym eq 'perllib_sep';
 }
 
 print $em <<'END';
