@@ -178,6 +178,7 @@ d_ftello='undef'
 d_ftime='undef'
 d_futimes='undef'
 d_futimesat='undef'
+d_gai_strerror='undef'
 d_gdbm_ndbm_h_uses_prototypes='undef'
 d_gdbmndbm_h_uses_prototypes='undef'
 d_getaddrinfo='undef'
@@ -378,6 +379,7 @@ d_pwpasswd='undef'
 d_pwquota='undef'
 d_qgcvt='undef'
 d_quad='undef'
+d_querylocale='undef'
 d_random_r='undef'
 d_readdir64_r='undef'
 d_readdir='define'
@@ -496,6 +498,7 @@ d_strcoll='undef'
 d_strctcpy='undef'
 d_strerrm='strerror(e)'
 d_strerror='define'
+d_strerror_l='undef'
 d_strerror_r='undef'
 d_strftime='undef'
 d_strlcat='undef'
@@ -724,6 +727,7 @@ i_values='undef'
 i_varargs='undef'
 i_varhdr='stdarg.h'
 i_vfork='undef'
+i_xlocale='undef'
 ignore_versioned_solibs='y'
 inc_version_list=''
 inc_version_list_init='0'
